@@ -11,6 +11,14 @@ from sklearn.metrics import confusion_matrix, auc
 from baal.utils.array_utils import to_prob
 
 
+def transpose_and_flatten(input):
+    if input.dim() > 2:
+        input = input.view(input.size(0), input.size(1), -1)  # N,C,H,W => N,C,H*W
+        input = input.transpose(1, 2)  # N,C,H*W => N,H*W,C
+        input = input.contiguous().view(-1, input.size(2))  # N,H*W,C => N*H*W,C
+    return input
+
+
 class Metrics(object):
     """
     metric is an abstract class.
@@ -98,7 +106,7 @@ class ECE(Metrics):
 
     def __init__(self, n_bins=10, **kwargs):
         self.n_bins = n_bins
-        self.tp, self.samples = None, None
+        self.tp, self.samples, self.conf_agg = None, None, None
         super().__init__(average=False)
 
     def update(self, output=None, target=None):
@@ -109,8 +117,9 @@ class ECE(Metrics):
             output (tensor): logits or predictions of model
             target (tensor): labels
         """
-        output = output.detach().cpu().numpy()
-        target = target.detach().cpu().numpy()
+
+        output = transpose_and_flatten(output).detach().cpu().numpy()
+        target = target.view([-1]).detach().cpu().numpy()
         output = to_prob(output)
 
         # this is to make sure handling 1.0 value confidence to be assigned to a bin
@@ -122,14 +131,15 @@ class ECE(Metrics):
             bin_id = int(math.floor(conf * self.n_bins))
             self.samples[bin_id] += 1
             self.tp[bin_id] += int(p_cls == t)
+            self.conf_agg[bin_id] += conf
 
     def _acc(self):
         return self.tp / np.maximum(1, self.samples)
 
     def calculate_result(self):
         n = self.samples.sum()
-        bin_confs = np.linspace(0, 1, self.n_bins)
-        return ((self.samples / n) * np.abs(self._acc() - bin_confs)).sum()
+        average_confidence = (self.conf_agg / np.maximum(self.samples, 1))
+        return ((self.samples / n) * np.abs(self._acc() - average_confidence)).sum()
 
     @property
     def value(self):
@@ -162,6 +172,7 @@ class ECE(Metrics):
     def reset(self):
         self.tp = np.zeros([self.n_bins])
         self.samples = np.zeros([self.n_bins])
+        self.conf_agg = np.zeros([self.n_bins])
 
 
 class ECE_PerCLs(Metrics):
@@ -181,6 +192,7 @@ class ECE_PerCLs(Metrics):
         self.n_cls = n_cls
         self.samples = np.zeros([self.n_cls, self.n_bins], dtype=int)
         self.tp = np.zeros([self.n_cls, self.n_bins], dtype=int)
+        self.conf_agg = np.zeros([self.n_cls, self.n_bins])
         super().__init__(average=False)
 
     def update(self, output=None, target=None):
@@ -198,11 +210,13 @@ class ECE_PerCLs(Metrics):
         # this is to make sure handling 1.0 value confidence to be assigned to a bin
         output = np.clip(output, 0, 0.9999)
 
-        for cls in range(self.n_cls):
-            for pred, t in zip(output[:, cls], target):
-                bin_id = int(math.floor(pred * self.n_bins))
-                self.samples[cls, bin_id] += 1
-                self.tp[cls, bin_id] += int(cls == t)
+        for pred, t in zip(output, target):
+            t = int(t)  # Force the conversion
+            conf, p_cls = pred.max(), pred.argmax()
+            bin_id = int(math.floor(conf * self.n_bins))
+            self.samples[t, bin_id] += 1
+            self.tp[t, bin_id] += int(p_cls == t)
+            self.conf_agg[t, bin_id] += conf
 
     def _acc(self):
         accuracy_per_class = np.zeros([self.n_cls, self.n_bins], dtype=float)
@@ -216,15 +230,16 @@ class ECE_PerCLs(Metrics):
         Returns:
             ece (nd.array): ece value per class
         """
-        bin_confs = np.linspace(0, 1, self.n_bins)
         accuracy = self._acc()
         ece = np.zeros([self.n_cls])
         for cls in range(self.n_cls):
             n = self.samples[cls, :].sum()
-            if n == 0 :
+            if n == 0:
                 ece[cls] = 0
             else:
-                ece[cls] = ((self.samples[cls, :] / n) * np.abs(accuracy[cls, :] - bin_confs)).sum()
+                bin_confidence = self.conf_agg[cls] / np.maximum(1, self.samples[cls])
+                diff_accuracy = np.abs(accuracy[cls, :] - bin_confidence)
+                ece[cls] = ((self.samples[cls, :] / n) * diff_accuracy).sum()
         return ece
 
     @property
@@ -259,6 +274,7 @@ class ECE_PerCLs(Metrics):
     def reset(self):
         self.tp = np.zeros([self.n_cls, self.n_bins])
         self.samples = np.zeros([self.n_cls, self.n_bins])
+        self.conf_agg = np.zeros([self.n_cls, self.n_bins])
 
 
 class Loss(Metrics):
